#include <algorithm>
#include <array>
#include <cassert>
#include <climits>
#include <cmath>
#include <iomanip>
#include <iostream>
#include <map>
#include <random>
#include <unordered_map>
#include <vector>
#include "circle.h"

using namespace std;

#ifndef LOCAL
#define cerr if(false) cerr
#endif // LOCAL

#define out(x) #x << " = " << x << "  "

const double EPS = 0.001;

std::mt19937 gen( 420 );
int randNum( int lb, int ub ){
    std::uniform_int_distribution< int > dist( lb, ub );
    return dist( gen );
}

double getDist( const std::pair< double, double >& a, const std::pair< double, double >& b ){
    double nas = ( a.first - b.first ) * ( a.first - b.first ) + ( a.second - b.second ) * ( a.second - b.second );
    return nas;
}

void solve( int k ){

    //std::set< std::pair< double, double > > stars;
    std::vector< std::pair< double, double > > stars;
    for( int i=0 ; i < 200 ; i++ ){
        auto curr = sample_star( );
        stars.push_back( curr );
    }

    for( auto j : stars ) cerr << j.first << " " << j.second << endl;
    cerr << endl;

    std::vector< std::pair< std::pair< double, double >, double > > nas;
    int cnt = 0;
    while( !stars.empty() ){
        int ind1 = randNum(0, stars.size() );
        int ind2 = randNum(0, stars.size() );
        int ind3 = randNum(0, stars.size() );

        std::pair< double, double > coord1 = stars[ind1];
        std::pair< double, double > coord2 = stars[ind2];
        std::pair< double, double > coord3 = stars[ind3];

        cerr << out( coord1.first ) << out( coord1.second ) << endl;
        cerr << out( coord2.first ) << out( coord2.second ) << endl;
        cerr << out( coord3.first ) << out( coord3.second ) << endl;



        double k11 = -2*coord1.first;
        double k12 = -2*coord1.second;
        double c1 = coord1.first * coord1.first + coord1.second * coord1.second;

        double k21 = -2*coord2.first;
        double k22 = -2*coord2.second;
        double c2 = coord2.first * coord2.first + coord2.second * coord2.second;

        double k31 = -2*coord3.first;
        double k32 = -2*coord3.second;
        double c3 = coord3.first * coord3.first + coord3.second * coord3.second;

        double KK11 = k11 - k21;
        double KK12 = k12 - k22;
        double CC1 = c2 - c1;

        double KK21 = k11 - k31;
        double KK22 = k12 - k32;
        double CC2 = c3 - c1;

        double del = KK11 / KK21;

        double KKK = KK12 - ( KK22 * del );
        double CCC = CC1 - ( CC2 * del );

        double Ay = CCC / KKK;

        // KK11 * Ax + KK12 * Ay = CC1
        // Ax = ( CC1 - KK12 * Ay ) / KK11
        double Ax = ( CC1 - KK12 * Ay ) / KK11;

        cerr << out( Ax ) << out( Ay ) << endl;

        std::pair< double, double > center;
        center.first = Ax; center.second = Ay;

        double currR = getDist( coord1, center );
        cerr << out( currR ) << endl;

        int currCnt = 0;
        std::vector< std::pair< double, double > > newStars;
        for( int i=0 ; i < stars.size() ; i++ ){
            auto j = stars[i];
            double currDist = getDist( center, j );
            //cerr << out( i ) << out( currDist ) << out( currDist - c) << endl;
            if( EPS > currDist - currR and currDist - currR > -EPS ){
                currCnt ++;
            }else{
                newStars.push_back( j );
            }
        }

        cerr << out( currCnt ) << endl;
        if( currCnt > 40 ){
            stars = newStars;
            nas.push_back({ center, currR });
        }

    }

    for( auto j : nas ){
        cerr << j.first.first << " " << j.first.second << " ; " << sqrt( j.second ) << endl;
        answer( j.first.first, j.first.second, sqrt( j.second ) );
    }

}
