#include "circle.h"
#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <cmath>
#include <map>

typedef long long llong;
const int MAXN = 500 + 12;
const double EPS = 0.001;
const int TAKE = 200;

double myabs(const double &x)
{
    if (x < 0.0) return -x;
    return x;
}

struct Point
{
    double x, y;
    inline friend bool operator < (const Point &a, const Point &b)
    {
        if (a.x != b.x) return a.x < b.x;
        return a.y < b.y;
    }
};


double dist(Point a, Point b)
{
    return sqrt((a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y));
}

double size(Point a, Point b, Point c)
{
    return myabs(a.x * b.y + a.y * c.x + b.x * c.y - b.y * c.x - a.x * c.y - a.y * b.x);
}

std::vector <std::pair <Point,double>> centers;
std::vector <Point> points;

struct Line
{
    double a, b;
};

Line getLine(Point a, Point b)
{
    return {(b.x - a.x) / (a.y - b.y), (a.x * a.x + a.y * a.y - b.x * b.x - b.y * b.y) / (2.0 * (a.y - b.y))};
}

Point intersect(Line first, Line second)
{
    double x = (second.b - first.b) / (first.a - second.a);
    double y = first.a * x + first.b;
    return {x, y}; 
}

std::pair <Point,double> get_center(Point a, Point b, Point c)
{
    double r = (dist(a, b) * dist(b, c)* dist(a, c) / (size(a, b, c) * 2.0)); 
    Line first = getLine(a, b);
    Line second = getLine(b, c);
    return {intersect(first, second), r};
}

void solve(int n)
{
    for (int i = 0 ; i < TAKE ; ++i)
    {
        auto [x, y] = sample_star();
        points.push_back({x, y});
    }

    for (int i = 0 ; i < TAKE ; ++i)
    {
        for (int j = i + 1 ; j < TAKE ; ++j)
        {
            for (int k = j + 1 ; k < TAKE ; ++k)
            {
                centers.push_back(get_center(points[i], points[j], points[k]));
            }
        }
    }

    std::sort(centers.begin(), centers.end());
    //stupid
    std::vector <std::pair <std::pair <Point,double>, int>> v;    
    for (const auto &p : centers)
    {
        if (v.size() && std::fabs(p.first.x - v.back().first.first.x) < EPS && std::fabs(p.first.y - v.back().first.first.y) < EPS)
        {
            v.back().second++;
            continue;
        }

        v.push_back({p, 1});
        // if (p.second > 3) std::cout << "here: " << p.first.first.x << ' ' << p.first.first.y << ' ' << p.first.second << ' ' << p.second << '\n';
    }

    std::sort(v.begin(), v.end(), [&](auto x, auto y)
    {
        return x.second > y.second;
    });

    for (int i = 0 ; i < n ; ++i)
    {
        // std::cout << "hsere: " << v[i].first.first.x << ' ' << v[i].first.first.y << ' ' << v[i].first.second << '\n';
        answer(v[i].first.first.x, v[i].first.first.y, v[i].first.second);
    }
}